//
// Copyright (c) Sindre Sorhus <sindresorhus@gmail.com> (https://sindresorhus.com)
// Released under the MIT license
// https://opensource.org/licenses/mit-license.php
//
/*!
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  Copyright (C) 2020 jeffy-g <hirotom1107@gmail.com>
  Released under the MIT license
  https://opensource.org/licenses/mit-license.php
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lcid = require("lcid");
const child_process_1 = require("child_process");
const defaultOptions = {
    spawn: true,
    cache: true
};
const defaultLocale = "en_US";
const execOpt = { encoding: "utf-8", timeout: 5000 };
const getStdOut = async (command, args) => await new Promise((resolve) => {
    child_process_1.execFile(command, args, execOpt, (err, stdout) => {
        resolve(err || stdout);
    });
});
const getStdOutSync = (command, args) => {
    try {
        return child_process_1.execFileSync(command, args, execOpt);
    }
    catch (e) {
        return e;
    }
};
const normalise = (input) => input.replace(/_/, "-");
function validate(result, processor) {
    if (typeof result === "string" && result.length) {
        return processor ? processor(result) : result.trim();
    }
    return defaultLocale;
}
const getEnvLocale = (env = process.env) => env.LC_ALL || env.LC_MESSAGES || env.LANG || env.LANGUAGE || "";
function parseLocale(str) {
    const env = str.split("\n").reduce((env, definition) => {
        const [key, value] = definition.split("=");
        if (key && value) {
            env[key] = value.replace(/^"|"$/g, "");
        }
        return env;
    }, {});
    return getEnvLocale(env);
}
const purgeExtraToken = (str) => (str && str.replace(/[.:].*/, "")) || defaultLocale;
const getSupportedLocale = (locale, locales) => locales.includes(locale) ? locale : defaultLocale;
const [getAppleLocale, getAppleLocaleSync] = ((cmd0, args0, cmd1, args1) => {
    return [
        async () => {
            const results = await Promise.all([
                getStdOut(cmd0, args0).then(ret => validate(ret)),
                getStdOut(cmd1, args1).then(ret => validate(ret))
            ]);
            return getSupportedLocale(results[0], results[1]);
        },
        () => getSupportedLocale(validate(getStdOutSync(cmd0, args0)), validate(getStdOutSync(cmd1, args1)))
    ];
})("defaults", ["read", "-globalDomain", "AppleLocale"], "locale", ["-a"]);
const getUnixLocale = async () => purgeExtraToken(parseLocale(await getStdOut("locale").then(ret => validate(ret))));
const getUnixLocaleSync = () => purgeExtraToken(parseLocale(validate(getStdOutSync("locale"))));
const parseLCID = (result) => {
    const lcidCode = parseInt(result.replace("Locale", ""), 16);
    return lcid.from(lcidCode);
};
const [getWinLocale, getWinLocaleSync] = ((cmd0, args0) => {
    return [
        async () => validate(await getStdOut(cmd0, args0), parseLCID),
        () => validate(getStdOutSync(cmd0, args0), parseLCID)
    ];
})("wmic", ["os", "get", "locale"]);
let detector;
{
    const syncFunctions = {
        win: getWinLocaleSync,
        mac: getAppleLocaleSync,
        unix: getUnixLocaleSync,
    };
    const asyncFunctions = {
        win: getWinLocale,
        mac: getAppleLocale,
        unix: getUnixLocale,
    };
    let cacheLocal;
    const base = (async) => (options = {}) => {
        options = { ...defaultOptions, ...options };
        const cache = options.cache;
        if (cache && (cacheLocal === null || cacheLocal === void 0 ? void 0 : cacheLocal.length)) {
            return (async ? Promise.resolve(cacheLocal) : cacheLocal);
        }
        const functions = async ? asyncFunctions : syncFunctions;
        let locale;
        const withCache = (l, mustPromise) => {
            l = normalise(l);
            cacheLocal = cache ? l : void 0;
            return (mustPromise ? Promise.resolve(l) : l);
        };
        const envLocale = getEnvLocale();
        if (envLocale || !options.spawn) {
            locale = purgeExtraToken(envLocale);
        }
        else if (process.platform === "win32") {
            locale = functions.win();
        }
        else if (process.platform === "darwin") {
            locale = functions.mac();
        }
        else {
            locale = functions.unix();
        }
        if (locale instanceof Promise) {
            return locale.then(result => withCache(result, false));
        }
        else {
            return withCache(locale, async);
        }
    };
    detector = base(true);
    detector.sync = base();
    Object.defineProperty(detector, "purge", {
        value: () => cacheLocal = void 0,
        enumerable: false,
    });
    Object.defineProperty(detector, "version", {
        value: "v1.0.2",
        enumerable: true,
    });
}
exports.osLocale = Object.freeze(detector);
